#!/bin/env python

import argparse
import collections
import json
import logging
import numpy as np
import gym
import gym.spaces
import gym.spaces.discrete as discrete
import sklearn.model_selection
import sys
import time

import pytrec_eval

import pyndri
import pyndri.utils


class NullAgent(object):

    @property
    def name(self):
        return 'null'

    def act(self, observation, reward, done, deterministic=True):
        return None

    def can_learn(self):
        return False

    def learn(self, env, *args, **kwargs):
        return None


class RandomAgent(object):

    def __init__(self, action_space):
        self.action_space = action_space

    @property
    def name(self):
        return 'random'

    def act(self, observation, reward, done, deterministic=True):
        return self.action_space.sample()

    def can_learn(self):
        return False

    def learn(self, env, *args, **kwargs):
        return None


class TabularQAgent(object):

    def __init__(self, observation_space, action_space, **userconfig):
        if not isinstance(observation_space, discrete.Discrete):
            raise RuntimeError()

        if not isinstance(action_space, discrete.Discrete):
            raise RuntimeError()

        self.observation_space = observation_space
        self.action_space = action_space
        self.action_n = action_space.n

        self.config = {
            'init_mean': 0.0,      # Initialize Q values with this mean
            'init_std': 0.0,       # Initialize Q values with this std dev
            'learning_rate': 0.1,
            'eps': 0.05,           # Epsilon in epsilon greedy policies
            'discount': 0.95,
            'n_iter': 1000,        # Number of iterations
        }

        self.config.update(userconfig)
        self.q = collections.defaultdict(
            lambda:
                self.config["init_std"] *
                np.random.randn(self.action_n) +
                self.config["init_mean"])

    @property
    def name(self):
        return 'tabular'

    def act(self, observation, reward, done, eps=None, deterministic=False):
        if not deterministic and eps is None:
            eps = self.config["eps"]
        elif deterministic:
            eps = -1.0

        if isinstance(observation, np.ndarray):
            observation = tuple(observation.tolist())
        elif isinstance(observation, tuple):
            pass
        else:
            raise RuntimeError()

        # epsilon greedy.
        action = np.argmax(self.q[observation]) \
            if np.random.random() > eps else self.action_space.sample()
        return action

    def can_learn(self):
        return True

    def learn(self, env, *args, **kwargs):
        total_reward = 0.0

        reward = 0.0
        done = False

        obs = env._reset(*args, **kwargs)
        obs = tuple(obs.tolist())

        q = self.q

        for t in range(self.config["n_iter"]):
            action = self.act(obs, reward, done)
            obs2, reward, done, _ = env.step(action)
            obs2 = tuple(obs2.tolist())

            if not done:
                future = np.max(q[obs2])
            else:
                future = 0.0

            q[obs][action] = \
                (1.0 - self.config['learning_rate']) * q[obs][action] + \
                self.config['learning_rate'] * (
                    reward + self.config['discount'] * future)

            obs = obs2

            total_reward += reward

            if t > 0 and t % 1000 == 0:
                logging.info('Step=%d, total reward=%.4f, average reward=%.4f',
                             t, total_reward, total_reward / t)

            if done:
                break

        return total_reward


class RetrievalEnv(gym.Env):

    def __init__(self, index,
                 measure='ndcg',
                 max_num_expanded_query_terms=None):
        vocabulary = pyndri.extract_dictionary(index)

        self.vocabulary = vocabulary

        self.action_space = gym.spaces.Discrete(len(self.vocabulary))
        self.observation_space = gym.spaces.Discrete(len(self.vocabulary))

        self.index = index
        self.measure = measure
        self.max_num_expanded_query_terms = max_num_expanded_query_terms

        assert self.max_num_expanded_query_terms is None or \
            self.max_num_expanded_query_terms > 0

        self.state = None  # Taken care of in reset().

    def _compute_utility(self, manyhot_query):
        query = sorted(np.nonzero(manyhot_query)[0])

        if len(query) >= 1:
            query_str = '#weight({})'.format(
                ' '.join(
                    '{:.2f} {}'.format(
                        manyhot_query[token_id],
                        self.vocabulary.id2token[token_id])
                    for token_id in query
                    if token_id > 0))
        else:
            query_str = None

        if not query_str:
            self.state['cache'][query_str] = {'0': {}}
        elif query_str not in self.state['cache']:
            document_scores = {
                self.index.ext_document_id(internal_doc_id): score
                for internal_doc_id, score in
                self.index.query(query_str, results_requested=10)}

            self.state['cache'][query_str] = {'0': document_scores}

        run = self.state['cache'][query_str]

        evaluation = self.state['evaluator'].evaluate(run)
        utility = evaluation['0'][self.measure]

        return utility

    def _step(self, action):
        if action > 0:
            next_manyhot_query = self.state['manyhot_query'].copy()
            next_manyhot_query[action] += 1

            prev_utility = self.state['utility']

            self.state['manyhot_query'] = next_manyhot_query
            self.state['utility'] = self._compute_utility(next_manyhot_query)

            reward = self.state['utility'] - prev_utility

            done = self.state['utility'] == 1.0
        else:
            next_manyhot_query = self.state['manyhot_query'].copy()
            reward = 0
            done = False

        self.state['num_steps'] += 1

        if self.max_num_expanded_query_terms and \
                self.state['num_steps'] >= \
                self.max_num_expanded_query_terms:
            done = True

        return next_manyhot_query > 0.0, reward, done, {}

    def _query_to_many_hot(self, query_token_ids):
        many_hot = np.zeros(len(self.vocabulary) + 1, dtype=np.int32)
        for token_idx in query_token_ids:
            many_hot[token_idx] = 1
        return many_hot

    def _reset(self, query_str, relevance):
        query_token_ids = [
            self.vocabulary.token2id[token]
            for token in self.index.tokenize(query_str)
            if token in self.vocabulary.token2id]
        manyhot_query = self._query_to_many_hot(query_token_ids)

        self.state = {
            'manyhot_query': manyhot_query,
            'evaluator': pytrec_eval.RelevanceEvaluator(
                {'0': relevance}, {self.measure}),
            'num_steps': 0,
            'cache': {},
        }

        self.original_utility = self._compute_utility(manyhot_query)
        self.state['utility'] = self.original_utility

        return self.state['manyhot_query'].copy()

    def _render(self, mode='human', close=False):
        return

    def _seed(self, seed=None):
        return []


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument('--index', type=pyndri.utils.existing_directory_path,
                        required=True)

    parser.add_argument('--limit_queries_for_debug',
                        type=pyndri.utils.positive_int,
                        default=None)

    parser.add_argument('--test_set_size', type=float, default=None)

    parser.add_argument('--num_epochs',
                        type=pyndri.utils.positive_int,
                        default=500)

    parser.add_argument('--queries', type=pyndri.utils.existing_file_path,
                        required=True)
    parser.add_argument('--query_relevance',
                        type=pyndri.utils.existing_file_path,
                        required=True)

    parser.add_argument('--trace_output',
                        type=pyndri.utils.nonexisting_file_path,
                        required=True)

    args = parser.parse_args()

    args.index = pyndri.Index(args.index)

    try:
        pyndri.utils.configure_logging(args)
    except IOError:
        return -1

    qrel = {}

    env = RetrievalEnv(args.index, max_num_expanded_query_terms=5)

    with open(args.queries, 'r') as f_queries:
        queries = list(pyndri.utils.read_queries(f_queries).items())

    with open(args.query_relevance, 'r') as f_qrel:
        qrel = pytrec_eval.parse_qrel(f_qrel)

    queries_idx = np.array(list(range(len(queries))))
    np.random.shuffle(queries_idx)

    if args.limit_queries_for_debug:
        queries_idx = queries_idx[:args.limit_queries_for_debug]

    if args.test_set_size and args.test_set_size > 0:
        train_queries_idx, test_queries_idx = \
            sklearn.model_selection.train_test_split(
                queries_idx, test_size=args.test_set_size)

        logging.info('Split query set into train=%s and test=%s.',
                     train_queries_idx.size, test_queries_idx.size)

        def evaluate(agent):
            episode_count = 1
            max_steps = 10

            logging.info('Evaluating %s using %d queries.',
                         agent, len(test_queries_idx))

            ndcgs = []

            for idx, query_idx in enumerate(test_queries_idx):
                reward = 0
                done = False

                for i in range(episode_count):
                    query_id, query_str = queries[query_idx]
                    ob = env._reset(query_str, qrel[query_id])

                    for _ in range(max_steps):
                        action = agent.act(ob, reward, done,
                                           deterministic=True)

                        if action is not None:
                            ob, reward, done, _ = env.step(action)

                        if done:
                            break

                    logging.debug('Query %s: %.4f -> %.4f',
                                  query_id,
                                  env.original_utility, env.state['utility'])

                    ndcgs.append(env.state['utility'])

                if idx > 0 and (idx + 1) % 10 == 0:
                    logging.info('Finished %d out of %d queries.',
                                 idx + 1, len(test_queries_idx))

            return ndcgs
    else:
        train_queries_idx = queries_idx

        def evaluate(agent):
            return np.nan,

    if args.trace_output:
        f_trace_out = open(args.trace_output, 'w')
    else:
        f_trace_out = None

    agents = [
        NullAgent(),
        RandomAgent(env.action_space),
        TabularQAgent(env.observation_space, env.action_space)]

    ndcg_per_agent = {}

    for agent in agents:
        if agent.can_learn():
            logging.info('Training %s using %d queries.',
                         agent, len(train_queries_idx))

            avg_rewards = []
            test_set_ndcgs = []

            start_time = time.time()

            for epoch_idx in range(args.num_epochs):
                logging.info('Epoch %d.', epoch_idx + 1)

                np.random.shuffle(train_queries_idx)

                avg_reward = 0.0

                for idx, query_idx in enumerate(train_queries_idx):
                    query_id, query_str = queries[query_idx]
                    relevance = qrel[query_id]

                    logging.debug('Learning from %s.', query_id)

                    total_reward = agent.learn(env, query_str, relevance)

                    if total_reward is not None:
                        avg_reward += total_reward

                    if idx > 0 and (idx + 1) % 500 == 0:
                        logging.info('Finished %d out of %d queries.',
                                     idx + 1, len(train_queries_idx))

                avg_reward /= len(train_queries_idx)
                avg_rewards.append(avg_reward)

                epoch_finish_time = time.time()

                epoch_data = {
                    'agent': agent.name,
                    'epoch_idx': epoch_idx,
                    'train_avg_reward': avg_reward,
                    'seconds_since_start': epoch_finish_time - start_time,
                }

                logging.info('Average rewards: %s', avg_rewards)

                if (epoch_idx + 1) % 10 == 0:
                    test_set_ndcg = np.mean(evaluate(agent))
                    test_set_ndcgs.append(test_set_ndcg)

                    logging.info('Test set NDCGs: %s', test_set_ndcgs)

                    epoch_data['test_set_ndcg'] = test_set_ndcg

                if f_trace_out:
                    f_trace_out.write(json.dumps(epoch_data))
                    f_trace_out.write('\n')

                    f_trace_out.flush()

        ndcgs = evaluate(agent)
        logging.info('NDCG: %.4f', np.mean(ndcgs))

        ndcg_per_agent[agent] = np.mean(ndcgs)

    logging.info('%s', ndcg_per_agent)

    if f_trace_out:
        f_trace_out.close()

if __name__ == '__main__':
    sys.exit(main())



"""Module pytrec_eval."""

__version__ = '0.5.9'

import collections
import re
from collections.abc import Callable, Iterable, Mapping
from typing import Dict, List, Set

import numpy as np
from pytrec_eval_ext import RelevanceEvaluator as _RelevanceEvaluator
from pytrec_eval_ext import supported_measures, supported_nicknames

__all__ = [
    'parse_run',
    'parse_qrel',
    'supported_measures',
    'supported_nicknames',
    'RelevanceEvaluator',
]


def parse_run(f_run: Iterable[str]) -> Dict[str, Dict[str, float]]:
    """Parse a TREC run file.

    Each line must be formatted as:
        <query_id> Q0 <doc_id> <rank> <score> <system_name>

    Args:
        f_run: An iterable of run file lines.

    Returns:
        A nested dictionary mapping query IDs to document IDs and scores:
        {
            "q1": {"d1": 2.5, "d2": 1.7, ...},
            "q2": {"d5": 0.8, ...},
        }

    Raises:
        AssertionError: If the same document ID appears twice for the same query.
    """
    run: Dict[str, Dict[str, float]] = collections.defaultdict(dict)
    for line in f_run:
        query_id, _, object_id, ranking, score, _ = line.strip().split()

        assert object_id not in run[query_id]
        run[query_id][object_id] = float(score)

    return run


def parse_qrel(f_qrel: Iterable[str]) -> Dict[str, Dict[str, int]]:
    """Parse a TREC qrel file.

    Each line must be formatted as:
        <query_id> 0 <doc_id> <relevance>

    Args:
        f_qrel: An iterable of qrel file lines.

    Returns:
        A nested dictionary mapping query IDs to document IDs and relevance levels:
        {
            "q1": {"d1": 1, "d2": 0, ...},
            "q2": {"d5": 2, ...},
        }

    Raises:
        AssertionError: If the same document ID appears twice for the same query.
    """
    qrel: Dict[str, Dict[str, int]] = collections.defaultdict(dict)
    for line in f_qrel:
        query_id, _, object_id, relevance = line.strip().split()

        assert object_id not in qrel[query_id]
        qrel[query_id][object_id] = int(relevance)

    return qrel


def compute_aggregated_measure(measure: str, values: List[float]) -> float:
    """Compute an aggregated evaluation measure across queries.

    The aggregation function is determined by the measure name:
        - If the measure starts with "num_", uses sum.
        - If the measure starts with "gm_", uses geometric mean.
        - Otherwise, uses arithmetic mean.

    Args:
        measure: The name of the measure (e.g., "map", "num_ret", "gm_map").
        values: A list of per-query measure values.

    Returns:
        The aggregated score across queries.
    """
    if measure.startswith('num_'):
        agg_fun: Callable[[List[float]], float] = np.sum
    elif measure.startswith('gm_'):

        def agg_fun(values: List[float]) -> float:
            return np.exp(np.sum(values) / len(values))
    else:
        agg_fun: Callable[[List[float]], float] = np.mean  # type: ignore[no-redef]
    return float(agg_fun(values))


class RelevanceEvaluator(_RelevanceEvaluator):
    def __init__(
        self,
        query_relevance: Mapping[str, Mapping[str, int]],
        measures: Iterable[str],
        relevance_level: int = 1,
        judged_docs_only_flag: bool = False,
    ) -> None:
        """Evaluate system runs against TREC-style relevance judgments.

        Args:
            query_relevance: A mapping from query IDs to document IDs and relevance levels.
            measures: A list or set of measures (or nicknames) to compute.
            relevance_level: Minimum relevance level considered relevant (default: 1).
            judged_docs_only_flag: If True, only judged documents are considered.
        """
        measures = self._expand_nicknames(measures)
        measures = self._combine_measures(measures)
        # fixes https://github.com/cvangysel/pytrec_eval/issues/57
        query_relevance = {query_id: qrels for query_id, qrels in query_relevance.items() if len(qrels) > 0}
        super().__init__(
            query_relevance=query_relevance,
            measures=measures,
            relevance_level=relevance_level,
            judged_docs_only_flag=judged_docs_only_flag,
        )

    def evaluate(self, scores: Mapping[str, Mapping[str, float]]) -> Dict[str, Dict[str, float]]:
        """Evaluate a run against the stored qrels.

        Args:
            scores: A mapping from query IDs to document IDs and scores.

        Returns:
            A nested dictionary mapping queries to measures to values:
            {
                "q1": {"map": 0.25, "ndcg": 0.4, ...},
                "q2": {...},
            }
            If `scores` is empty, returns an empty dict.
        """
        if not scores:
            return {}
        return super().evaluate(scores)

    def _expand_nicknames(self, measures: Iterable[str]) -> Set[str]:
        """Expand measure nicknames into their constituent measures."""
        result = set()
        for measure in measures:
            if measure in supported_nicknames:
                result.update(supported_nicknames[measure])
            else:
                result.add(measure)
        return result

    def _combine_measures(self, measures: Iterable[str]) -> Set[str]:
        """Normalize and combine measures into TREC-eval compatible format.

        Handles:
          - Non-parameterized measures: "map"
          - Single-parameter measures: "ndcg.10" or "ndcg_10"
          - Multi-parameter measures: "ndcg.5,10" or "ndcg_5,10"

        Args:
            measures: An iterable of measure strings.

        Returns:
            A set of normalized measure strings in "measure.param1,param2" format.

        Raises:
            ValueError: If an unsupported measure is encountered.
        """
        RE_BASE = r'{}[\._]([0-9]+(\.[0-9]+)?(,[0-9]+(\.[0-9]+)?)*)'

        # break apart measures in any of the following formats and combine
        #  1) meas          -> {meas: {}}  # either non-parameterized measure or use default params
        #  2) meas.p1       -> {meas: {p1}}
        #  3) meas_p1       -> {meas: {p1}}
        #  4) meas.p1,p2,p3 -> {meas: {p1, p2, p3}}
        #  5) meas_p1,p2,p3 -> {meas: {p1, p2, p3}}
        param_meas: dict[str, set[str]] = collections.defaultdict(set)
        for measure in measures:
            if measure not in supported_measures and measure not in supported_nicknames:
                matches = ((m, re.match(RE_BASE.format(re.escape(m)), measure)) for m in supported_measures)
                match = next(filter(lambda x: x[1] is not None, matches), None)
                if match is None:
                    raise ValueError(f'unsupported measure {measure}')
                base_meas, meas_args = match[0], match[1].group(1)  # type: ignore[union-attr]
                param_meas[base_meas].update(meas_args.split(','))
            elif measure not in param_meas:
                param_meas[measure] = set()

        # re-construct in meas.p1,p2,p3 format for trec_eval
        fmt_meas = set()
        for meas, meas_args in param_meas.items():
            if meas_args:
                meas = '{}.{}'.format(meas, ','.join(sorted(meas_args)))
            fmt_meas.add(meas)

        return fmt_meas



"""Demonstrates how statistical significance tests can be ran using pytrec_eval."""

import argparse
import os
import scipy.stats
import sys

import pytrec_eval


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument('qrel')
    parser.add_argument('run', nargs=2)

    # A bit too strict, as it does not allow for parametrized measures,
    # but sufficient for the example.
    parser.add_argument('--measure',
                        choices=pytrec_eval.supported_measures,
                        required=True)

    args = parser.parse_args()

    assert os.path.exists(args.qrel)
    assert all(map(os.path.exists, args.run))

    with open(args.qrel, 'r') as f_qrel:
        qrel = pytrec_eval.parse_qrel(f_qrel)

    with open(args.run[0], 'r') as f_run:
        first_run = pytrec_eval.parse_run(f_run)

    with open(args.run[1], 'r') as f_run:
        second_run = pytrec_eval.parse_run(f_run)

    evaluator = pytrec_eval.RelevanceEvaluator(
        qrel, {args.measure})

    first_results = evaluator.evaluate(first_run)
    second_results = evaluator.evaluate(second_run)

    query_ids = list(
        set(first_results.keys()) & set(second_results.keys()))

    first_scores = [
        first_results[query_id][args.measure] for query_id in query_ids]
    second_scores = [
        second_results[query_id][args.measure] for query_id in query_ids]

    print(scipy.stats.ttest_rel(first_scores, second_scores))

if __name__ == "__main__":
    sys.exit(main())



"""Approximately simulates trec_eval using pytrec_eval."""

import argparse
import os
import sys

import pytrec_eval


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument('qrel')
    parser.add_argument('run')

    args = parser.parse_args()

    assert os.path.exists(args.qrel)
    assert os.path.exists(args.run)

    with open(args.qrel, 'r') as f_qrel:
        qrel = pytrec_eval.parse_qrel(f_qrel)

    with open(args.run, 'r') as f_run:
        run = pytrec_eval.parse_run(f_run)

    evaluator = pytrec_eval.RelevanceEvaluator(
        qrel, pytrec_eval.supported_measures)

    results = evaluator.evaluate(run)

    def print_line(measure, scope, value):
        print('{:25s}{:8s}{:.4f}'.format(measure, scope, value))

    for query_id, query_measures in sorted(results.items()):
        for measure, value in sorted(query_measures.items()):
            print_line(measure, query_id, value)

    # Scope hack: use query_measures of last item in previous loop to
    # figure out all unique measure names.
    #
    # TODO(cvangysel): add member to RelevanceEvaluator
    #                  with a list of measure names.
    for measure in sorted(query_measures.keys()):
        print_line(
            measure,
            'all',
            pytrec_eval.compute_aggregated_measure(
                measure,
                [query_measures[measure]
                 for query_measures in results.values()]))

if __name__ == "__main__":
    sys.exit(main())



"""A very simple example, but then with parametrized measures."""

import pytrec_eval
import json

qrel = {
    'q1': {
        'd1': 0,
        'd2': 1,
        'd3': 0,
    },
    'q2': {
        'd2': 1,
        'd3': 1,
    },
}

run = {
    'q1': {
        'd1': 1.0,
        'd2': 0.0,
        'd3': 1.5,
    },
    'q2': {
        'd1': 1.5,
        'd2': 0.2,
        'd3': 0.5,
    }
}

evaluator = pytrec_eval.RelevanceEvaluator(
    qrel, {'map_cut', 'ndcg_cut'})

print(json.dumps(evaluator.evaluate(run), indent=1))



"""Sets up pytrec_eval."""

from setuptools import setup, Extension
import os
import sys
import tempfile
from pathlib import Path

REMOTE_TREC_EVAL_URI = 'https://github.com/usnistgov/trec_eval/archive/v9.0.8.tar.gz'

REMOTE_TREC_EVAL_TLD_NAME = 'trec_eval-9.0.8'

LOCAL_TREC_EVAL_DIR = os.path.realpath(
    os.path.join(__file__, '..', 'trec_eval'))

TREC_EVAL_SRC = []

import codecs
def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()

def get_version(rel_path):
    import os
    suffix = os.environ["PYTRECEVAL_VERSION_SUFFIX" ] if "PYTRECEVAL_VERSION_SUFFIX" in os.environ else ""
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1] + suffix
    else:
        raise RuntimeError("Unable to find version string.")

def get_name():
    suffix = os.environ["PYTRECEVAL_NAME_SUFFIX" ] if "PYTRECEVAL_NAME_SUFFIX" in os.environ else ""
    return "pytrec_eval-terrier" + suffix

with tempfile.TemporaryDirectory() as tmp_dir:
    if os.path.isfile(os.path.join(LOCAL_TREC_EVAL_DIR, 'trec_eval.h')):
        # Use local version.
        trec_eval_dir = LOCAL_TREC_EVAL_DIR
    else:  # Fetch remote version.
        print('Fetching trec_eval from {}.'.format(REMOTE_TREC_EVAL_URI))

        import io
        import urllib.request

        response = urllib.request.urlopen(REMOTE_TREC_EVAL_URI)
        mmap_f = io.BytesIO(response.read())

        if REMOTE_TREC_EVAL_URI.endswith('.zip'):
            import zipfile

            trec_eval_archive = zipfile.ZipFile(mmap_f)
        elif REMOTE_TREC_EVAL_URI.endswith('.tar.gz'):
            import tarfile

            trec_eval_archive = tarfile.open(fileobj=mmap_f)

        trec_eval_archive.extractall(tmp_dir)

        trec_eval_dir = os.path.join(tmp_dir, REMOTE_TREC_EVAL_TLD_NAME)

    for filename in os.listdir(trec_eval_dir):
        if filename.endswith('.c') and not filename == "trec_eval.c":
            TREC_EVAL_SRC.append(os.path.join(trec_eval_dir, filename))
    #include the windows/ subdirectory on windows machines
    if sys.platform == 'win32':
        for filename in os.listdir(os.path.join(trec_eval_dir, "windows")):
            if filename.endswith('.c') and not filename == "trec_eval.c":
                TREC_EVAL_SRC.append(os.path.join(trec_eval_dir, "windows", filename))

    pytrec_eval_ext = Extension(
        'pytrec_eval_ext',
        sources=['src/pytrec_eval.cpp'] + TREC_EVAL_SRC,
        #windows doesnt need libm
        libraries=[] if sys.platform == 'win32' else ['m', 'stdc++'],
        include_dirs=[trec_eval_dir, os.path.join(trec_eval_dir, "windows")] if sys.platform == 'win32' else [trec_eval_dir],
        undef_macros=['NDEBUG'],
        extra_compile_args=['-g', '-Wall', '-O3'],
        define_macros=[('VERSIONID', '\"pytrec_eval\"'),
                       ('_GLIBCXX_USE_CXX11_ABI', '0'),
                       ('P_NEEDS_GNU_CXX_NAMESPACE', '1')])

    requirements_file = Path(__file__).parent / 'requirements.txt'
    with requirements_file.open('r') as fp:
        requirements = fp.read().splitlines()

    setup(name=get_name(),
          version=get_version("py/__init__.py"),
          description='Provides Python bindings for popular '
                      'Information Retrieval measures implemented '
                      'within trec_eval.',
          install_requires=requirements,
          author='Christophe Van Gysel -> forked by Terrier team',
          author_email='cvangysel@uva.nl',
          ext_modules=[pytrec_eval_ext],
          packages=['pytrec_eval'],
          package_dir={'pytrec_eval': 'py'},
          python_requires='>=3',
          url='https://github.com/terrierteam/pytrec_eval',
          download_url='https://github.com/terrierteam/pytrec_eval/releases',
          keywords=[
              'trec_eval',
              'information retrieval',
              'evaluation',
              'ranking',
          ],
          classifiers=[
              'Development Status :: 3 - Alpha',
              'License :: OSI Approved :: MIT License',
              'Programming Language :: Python',
              'Programming Language :: C++',
              'Intended Audience :: Science/Research',
              'Operating System :: POSIX :: Linux',
              'Topic :: Text Processing :: General',
          ])
